@@ -1,34 +1,44 @@
 import os
 from getpass import getpass
 
-from cloudmesh.burn.burner import Burner
-from cloudmesh.burn.burner import MultiBurner
-from cloudmesh.burn.burner import gen_strong_pass
+# from cloudmesh.common.debug import VERBOSE
+from cloudmesh.burn.Imager import Imager
+from cloudmesh.burn.burner.Burner import Burner
+from cloudmesh.burn.burner.raspberryos import MultiBurner
 from cloudmesh.burn.image import Image
 from cloudmesh.burn.network import Network
-from cloudmesh.burn.util import os_is_pi
-from cloudmesh.burn.util import os_is_mac
+from cloudmesh.burn.sdcard import SDCard
 from cloudmesh.burn.util import os_is_linux
+from cloudmesh.burn.util import os_is_mac
+from cloudmesh.burn.util import os_is_pi
 from cloudmesh.common.StopWatch import StopWatch
 from cloudmesh.common.Tabulate import Printer
 from cloudmesh.common.parameter import Parameter
+from cloudmesh.common.security import generate_strong_pass
 from cloudmesh.common.util import Console
+from cloudmesh.common.util import yn_choice
 from cloudmesh.shell.command import PluginCommand
 from cloudmesh.shell.command import command
 from cloudmesh.shell.command import map_parameters
-# from cloudmesh.common.debug import VERBOSE
-from cloudmesh.burn.Imager import Imager
-from cloudmesh.common.util import yn_choice
+from cloudmesh.burn.usb import USB
+from cloudmesh.common.debug import VERBOSE
 
 
 class BurnCommand(PluginCommand):
 
+    # noinspection PyBroadException
     @command
     def do_burn(self, args, arguments):
         """
         ::
 
             Usage:
+              burn gui [--hostname=HOSTNAME]
+                       [--ip=IP]
+                       [--ssid=SSID]
+                       [--wifipassword=PSK]
+                       [--bs=BLOCKSIZE]
+                       [--dryrun]
               burn firmware check
               burn firmware update
               burn install
@@ -40,7 +50,7 @@ class BurnCommand(PluginCommand):
               burn network list [--ip=IP] [--used]
               burn network
               burn info [--device=DEVICE]
-              burn image versions [--refresh] [--yaml]
+              burn image versions [--details] [--refresh] [--yaml]
               burn image ls
               burn image delete [--image=IMAGE]
               burn image get [--url=URL] [TAG...]
@@ -48,19 +58,22 @@ class BurnCommand(PluginCommand):
               burn copy [--device=DEVICE] [--from=DESTINATION]
               burn shrink [--image=IMAGE]
               burn cluster --device=DEVICE --hostname=HOSTNAME
+                           [--burning=BURNING]
                            [--ip=IP]
                            [--ssid=SSID]
                            [--wifipassword=PSK]
                            [--bs=BLOCKSIZE]
+                           [--os=OS]
                            [-y]
-                           [-g]
+                           [--imaged]
+                           [--set_passwd]
               burn create [--image=IMAGE]
                           [--device=DEVICE]
+                          [--burning=BURNING]
                           [--hostname=HOSTNAME]
                           [--ip=IP]
                           [--sshkey=KEY]
                           [--blocksize=BLOCKSIZE]
-                          [--dryrun]
                           [--passwd=PASSWD]
                           [--ssid=SSID]
                           [--wifipassword=PSK]
@@ -69,7 +82,7 @@ class BurnCommand(PluginCommand):
                           [--inventory=INVENTORY]
                           [--name=NAME]
                           [-y]
-              burn sdcard [TAG...] [--device=DEVICE] [--dryrun]
+              burn sdcard [TAG...] [--device=DEVICE] [-y]
               burn set [--hostname=HOSTNAME]
                        [--ip=IP]
                        [--key=KEY]
@@ -79,20 +92,23 @@ class BurnCommand(PluginCommand):
               burn check [--device=DEVICE]
               burn mac --hostname=HOSTNAME
 
+
+
             Options:
               -h --help              Show this screen.
               --version              Show version.
               --image=IMAGE          The image filename,
                                      e.g. 2019-09-26-raspbian-buster.img
               --device=DEVICE        The device, e.g. /dev/sdX
-              --hostname=HOSTNAME    The hostname
-              --ip=IP                The IP address
+              --hostname=HOSTNAME    The hostnames of the cluster
+              --ip=IP                The IP addresses of the cluster
               --key=KEY              The name of the SSH key file
               --blocksize=BLOCKSIZE  The blocksise to burn [default: 4M]
+              --burning=BURNING      The hosts to be burned
 
             Arguments:
-                TAG                  Keyword tags to identify an image
-                                     [default: latest]
+               TAG                   Keyword tags to identify an image
+
             Files:
               This is not fully thought through and needs to be documented
               ~/.cloudmesh/images
@@ -243,7 +259,6 @@ class BurnCommand(PluginCommand):
                                 [--ip=IP]
                                 [--sshkey=KEY]
                                 [--blocksize=BLOCKSIZE]
-                                [--dryrun]
                                 [--passwd=PASSWD]
                                 [--ssid=SSID]
                                 [--wifipassword=PSK]
@@ -252,7 +267,7 @@ class BurnCommand(PluginCommand):
                     This command  not only can format the SDCard, but
                     also initializes it with specific values
 
-                cms burn sdcard [TAG...] [--device=DEVICE] [--dryrun]
+                cms burn sdcard [TAG...] [--device=DEVICE]
 
                     this burns the sd card, see also copy and create
 
@@ -305,18 +320,19 @@ class BurnCommand(PluginCommand):
                > cms burn image delete 2019-09-26-raspbian-buster-lite
 
         """
-
         map_parameters(arguments,
+                       "details",
                        "refresh",
                        "device",
+                       "dryrun",
+                       "burning",
                        "hostname",
                        "ip",
                        "sshkey",
                        "blocksize",
-                       "dryrun",
-                       # "output",
                        "ssid",
                        "url",
+                       "imaged",
                        "key",
                        "keyboard",
                        "passwd",
@@ -327,17 +343,18 @@ class BurnCommand(PluginCommand):
                        "country",
                        "inventory",
                        "name",
-                       "bs")
+                       "bs",
+                       "set_passwd")
 
-        # VERBOSE(arguments)
         # arguments.MOUNTPOINT = arguments["--mount"]
         arguments.FORMAT = arguments["--format"]
         arguments.FROM = arguments["--from"]
         arguments.IMAGE = arguments["--image"]
         arguments.output = "table"  # hard code for now
         arguments.bs = arguments.bs or "4M"
-        arguments.gui = arguments["-g"]
         arguments.yes = arguments["-y"]
+        if len(arguments.TAG) == 0:
+            arguments.TAG = "latest"
 
         # VERBOSE(arguments)
 
@@ -348,12 +365,8 @@ class BurnCommand(PluginCommand):
             StopWatch.status(label, True)
             return result
 
-        dryrun = arguments['--dryrun']
-
-        StopWatch.start("info")
-        burner = Burner(dryrun=dryrun)
-        StopWatch.stop("info")
-        StopWatch.status("info", True)
+        burner = Burner()
+        sdcard = SDCard()
 
         if arguments.imager:
 
@@ -362,7 +375,7 @@ class BurnCommand(PluginCommand):
             Console.msg(f"Tags: {arguments.TAG}")
             try:
                 file = Imager.fetch(tag=arguments.TAG)
-            except:
+            except:  # noqa: E722
                 pass
 
             try:
@@ -372,6 +385,16 @@ class BurnCommand(PluginCommand):
 
             return ""
 
+        elif arguments.gui:
+
+            from cloudmesh.burn.gui import Gui
+            VERBOSE(arguments)
+            g = Gui(hostname=arguments.hostname, ip=arguments.ip, dryrun=arguments.dryrun)
+
+            g.run()
+
+            return ""
+
         elif arguments.firmware and arguments.check:
 
             execute("firmware check", burner.firmware(action="check"))
@@ -397,24 +420,25 @@ class BurnCommand(PluginCommand):
             if arguments["--yaml"]:
                 output = "yaml"
 
-            print(Printer.write(
-                result,
-                order=["tag", 'date', "type", 'version', "url"],
-                header=["Tag", 'Date', "Type", 'Version', "Url"],
-                output=output
-            )
-            )
+            order = ["tag", 'date', "os", "type", 'version']
+            header = ["Tag", 'Date', "OS", "Type", 'Version']
+            if arguments.details:
+                order = ["tag", 'date', "os", "type", 'version', "url"]
+                header = ["Tag", 'Date', "OS", "Type", 'Version', "Url"]
+
+            print(Printer.write(result, order=order, header=header, output=output))
 
             StopWatch.stop("image versions")
             StopWatch.status("image versions", True)
             return ""
 
         elif arguments.load:
-            execute("load", burner.load_device(device=arguments.device))
+            execute("load", sdcard.load_device(device=arguments.device))
             return ""
 
         elif arguments["format"]:  # as format is a python word, we need to use an index
-            execute("format", burner.format_device(device=arguments.device, unmount=True))
+
+            execute("format", sdcard.format_device(device=arguments.device, unmount=True))
             return ""
 
         elif arguments.network and arguments["list"]:
@@ -484,7 +508,16 @@ class BurnCommand(PluginCommand):
         elif arguments.info:
 
             output = arguments.output or "table"
-            execute("info", burner.info(output=output))
+            card = SDCard()
+            execute("info", card.info(output=output))
+            try:
+                USB.check_for_readers()
+            except Exception as e:
+                print()
+                Console.error(e)
+                print()
+                return ""
+
             return ""
 
         elif arguments.install:
@@ -506,27 +539,59 @@ class BurnCommand(PluginCommand):
             return ""
 
         elif arguments.backup:
-            execute("backup", burner.backup(device=arguments.device, to_file=arguments.to))
+            try:
+                USB.check_for_readers()
+            except Exception as e:
+                print()
+                Console.error(e)
+                print()
+                return ""
+            execute("backup", sdcard.backup(device=arguments.device, to_file=arguments.to))
             return ""
 
         elif arguments["copy"]:  # as copy is a reserved word we need to use the index
-            execute("copy", burner.copy(device=arguments.device, from_file=arguments.FROM))
+            USB.check_for_readers()
+            execute("copy", sdcard.copy(device=arguments.device, from_file=arguments.FROM))
             return ""
 
         elif arguments.sdcard:
+
+            try:
+                USB.check_for_readers()
+            except Exception as e:
+                print()
+                Console.error(e)
+                print()
+                return ""
+
+            if arguments.device is None:
+                card = SDCard()
+                card.info()
+                Console.error("Please specify a device")
+                return ""
+
             arguments.TAG = arguments.TAG or ["latest-lite"]
 
-            execute("sdcard", burner.burn_sdcard(tag=arguments.TAG, device=arguments.device))
+            execute("format", sdcard.format_device(device=arguments.device, unmount=True))
+            execute("unmount", sdcard.unmount(device=arguments.device))
+
+            execute("sdcard", sdcard.burn_sdcard(tag=arguments.TAG, device=arguments.device, yes=arguments.yes))
             return ""
 
         elif arguments.mount:
-            execute("mount", burner.mount(device=arguments.device,
-                                          card_os=arguments.os))
+
+            if arguments.device is None:
+                card = SDCard
+                card.info()
+                Console.error("Please specify a device")
+                return ""
+
+            execute("mount", sdcard.mount(device=arguments.device, card_os=arguments.os))
             return ""
 
         elif arguments.unmount:
-            execute("unmount", burner.unmount(device=arguments.device,
-                                              card_os=arguments.os))
+
+            execute("unmount", sdcard.unmount(device=arguments.device, card_os=arguments.os))
             return ""
 
         elif arguments.mac:
@@ -538,6 +603,14 @@ class BurnCommand(PluginCommand):
 
         elif arguments.set:
 
+            try:
+                USB.check_for_readers()
+            except Exception as e:
+                print()
+                Console.error(e)
+                print()
+                return ""
+
             if arguments.hostname:
                 execute("set hostname", burner.set_hostname(arguments.hostname))
 
@@ -548,11 +621,18 @@ class BurnCommand(PluginCommand):
                 execute("set key", burner.set_key(arguments.key))
 
             if arguments.keyboard:
-                execute("set keyboard", burner.set_keyboard(country=arguments.keyboard))
+                execute("set keyboard", burner.keyboard(country=arguments.keyboard))
 
             return ""
 
         elif arguments.enable and arguments.ssh:
+            try:
+                USB.check_for_readers()
+            except Exception as e:
+                print()
+                Console.error(e)
+                print()
+                return ""
 
             execute("enable ssh", burner.enable_ssh())
             return ""
@@ -599,13 +679,29 @@ class BurnCommand(PluginCommand):
             #                  --wifipassword=mypass
             #
 
+            try:
+                USB.check_for_readers()
+            except Exception as e:
+                print()
+                Console.error(e)
+                print()
+                return ""
             execute("cluster", burner.cluster(arguments=arguments))
             return ""
 
         elif arguments.create and arguments.inventory:
+            try:
+                USB.check_for_readers()
+            except Exception as e:
+                print()
+                Console.error(e)
+                print()
+                return ""
+
             if not os_is_pi():
                 print()
-                Console.error("This command has only been written for a  Raspberry Pis. Terminating for caution")
+                Console.error("This command has only been written for a  Raspberry Pis. "
+                              "Terminating for caution")
                 print()
                 if yn_choice("Continue anyways?"):
                     pass
@@ -626,7 +722,8 @@ class BurnCommand(PluginCommand):
                 inventory=inventory,
                 name=arguments.name,
                 device=arguments.device,
-                yes=arguments.yes
+                yes=arguments.yes,
+                passwd=arguments.passwd
             )
             StopWatch.stop("burn inventory")
             StopWatch.status("burn inventory", True)
@@ -635,12 +732,20 @@ class BurnCommand(PluginCommand):
 
         elif arguments.create:
 
+            try:
+                USB.check_for_readers()
+            except Exception as e:
+                print()
+                Console.error(e)
+                print()
+                return ""
+
             if arguments["--passwd"]:
                 passwd = arguments["--passwd"]
             elif "PASSWD" in os.environ:
                 passwd = os.environ["PASSWD"]
             else:
-                passwd = gen_strong_pass()
+                passwd = generate_strong_pass()
 
             psk = None
             if arguments["--ssid"]:
@@ -656,8 +761,6 @@ class BurnCommand(PluginCommand):
                 else:
                     ssid = None
 
-            # check_root(dryrun=dryrun)
-
             image = 'latest' or arguments.IMAGE
 
             dev = os.environ['DEV'] if 'DEV' in os.environ else None
@@ -668,6 +771,13 @@ class BurnCommand(PluginCommand):
 
             hostnames = Parameter.expand(arguments.hostname)
 
+            if arguments.burnimg is None:
+                burning = hostnames
+            else:
+                burning = arguments.burning
+
+            VERBOSE(arguments)
+
             ips = None if not arguments.ip else Parameter.expand(arguments.ip)
             key = arguments.sshkey
             tag = arguments['--tag']
@@ -678,6 +788,7 @@ class BurnCommand(PluginCommand):
 
                 multi = MultiBurner()
                 multi.burn_all(
+                    burning=burning,
                     image=image,
                     device=devices,
                     blocksize=blocksize,
